using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace PointScatter
{
    public partial class Form1 : Form
    {
        private const int numParticles = 400;
        private Particle[] particles = new Particle[numParticles];

        private Random random = new Random();

        public Form1()
        {
            InitializeComponent();

            for (int i = 0; i < numParticles; i++)
            {
                particles[i].position = new PointF(random.Next(ClientSize.Width), random.Next(ClientSize.Height));
            }
        }

        // Actually, this is not physics.
        // In physics, the force affects the velocity, not the position directly.
        private void doPhysics()
        {
            for (int i = 0; i < numParticles; i++)
            {
                double fx = 0.0;
                double fy = 0.0;

                double mx = mouseX - particles[i].position.X;
                double my = mouseY - particles[i].position.Y;

                double distance = Math.Sqrt(mx * mx + my * my);
                double force = 0.0;
                if (distance < 100.0)
                {
                    // The force decays to 0 as the distance tends toward 70.
                    force = - 100.0 / (0.5 + distance / 100.0) + 100.0 / 1.5;
                    force = Math.Min(force, 50.0);
                }

                fx += force * mx / (0.001 + distance);
                fy += force * my / (0.001 + distance);

                for (int j = 0; j < numParticles; j++)
                {
                    if (j == i)
                    {
                        continue;
                    }
                    double dx = particles[j].position.X - particles[i].position.X;
                    double dy = particles[j].position.Y - particles[i].position.Y;
                    distance = Math.Sqrt(dx * dx + dy * dy);
                    force = 0.0;
                    if (distance < 20.0)
                    {
                        force = - 100.0 / ((0.1 + distance/20.0)*(0.1 + distance/20.0)) + 100.0 / (1.1 * 1.1);
                    }
                    else
                    {
                        force = 0.5 * Math.Sqrt(distance - 20.0) / distance;
                    }
                    force = Math.Min(force, 50.0);

                    fx += force * dx / (0.001 + distance);
                    fy += force * dy / (0.001 + distance);
                }

                particles[i].position.X += (float)(0.03 * fx);
                particles[i].position.Y += (float)(0.03 * fy);
            }
        }

        private int mouseX = -1000;
        private int mouseY = -1000;

        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            mouseX = e.X;
            mouseY = e.Y;
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);

            doPhysics();

            for (int i = 0; i < numParticles; i++)
            {
                particles[i].Draw(e.Graphics);
            }

            Invalidate();
        }

    }

    internal struct Particle
    {
        internal PointF position;

        internal void Draw (Graphics g)
        {
            g.FillEllipse(Brushes.Black, position.X-5.0f, position.Y-5.0f, 10.0f, 10.0f); 
        }
    }
}